/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.dao;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.hibernate.Criteria;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.Subqueries;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import ch.semafor.gendas.model.Element;
import ch.semafor.gendas.model.Modification;
import ch.semafor.gendas.model.PropertyValue;

@Repository("elementDao")
public class ElementDaoImpl extends GenericDaoJpa<Element, Long> implements
		ElementDao {
	private final Logger logger = LoggerFactory.getLogger(ElementDaoImpl.class);

	ElementDaoImpl() {
		super(Element.class);
	}

	/*
	 * @SuppressWarnings("unchecked") public Element findElementByName(String
	 * elementName) throws ElementNotFoundException { List<Element> l =
	 * getEntityManager() .createQuery(
	 * "from Element where name = :name and parent is null") .
	 * setParameter("name", elementName) .getResultList(); if (l == null ||
	 * l.isEmpty()) throw new ElementNotFoundException("element \''" + elementName
	 * + "'\' not found..."); return l.get(0); }
	 */

	public List<Element> findByType(final String type) {
		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("type", type);
		return findByNamedQuery("Element.findByType", queryParams);
	}

	public List<Element> findByTypeAndReference(final String type,
			final Element ref) {
		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("ref", ref);
		queryParams.put("type", type);
		return findByNamedQuery("Element.findByTypeAndReference", queryParams);
	}

	public List<PropertyValue> findPropertyTypeAndStringLike(final String ptype,
			final String value) {
		Criteria crit = createCriteria(PropertyValue.class);
		crit.add(Restrictions.ilike("svalue", value.toLowerCase())).createCriteria( // NOPMD
																																								// by
																																								// wim
																																								// on
																																								// 9/20/10
																																								// 9:05
																																								// AM
				"valuelist")
				.add(Restrictions.eq("nextRevision", Modification.MaxRevision));
		return crit.list();
	}

	public List<Element> findByTypeAndPropertyTypeAndStringLike(
			final String etype, final String ptype, final String value) {

		Map<String, Object> queryParams = new HashMap<String, Object>();
		String v = ""; // NOPMD by wim on 9/20/10 8:44 AM
		if (value != null) {
			v = value.toLowerCase();
		}
		queryParams.put("v", v);
		queryParams.put("ptype", ptype);
		queryParams.put("etype", etype);
		return findByNamedQuery("Element.findByTypeAndPropertyTypeAndStringLike",
				queryParams);
	}

	public int getCountByTypeAndPropertyTypeAndStringLike(final String etype,
			final String ptype, final String value) {

		Map<String, Object> queryParams = new HashMap<String, Object>();
		String v = ""; // NOPMD by wim on 9/20/10 8:44 AM
		if (value != null) {
			v = value.toLowerCase();
		}
		queryParams.put("v", v);
		queryParams.put("ptype", ptype);
		queryParams.put("etype", etype);
		return countByNamedQuery(
				"Element.getCountByTypeAndPropertyTypeAndStringLike", queryParams);
	}

	public int getCountByTypeAnd2PropertyTypesAndStringsLike(final String etype,
			final String ptype1, final String value1, final String ptype2,
			final String value2) {

		Map<String, Object> queryParams = new HashMap<String, Object>();
		String v1 = ""; // NOPMD by wim on 9/20/10 8:44 AM
		if (value1 != null) {
			v1 = value1.toLowerCase();
		}
		String v2 = ""; // NOPMD by wim on 9/20/10 8:44 AM
		if (value2 != null) {
			v2 = value2.toLowerCase();
		}
		queryParams.put("v1", v1);
		queryParams.put("ptype1", ptype1);
		queryParams.put("v2", v2);
		queryParams.put("ptype2", ptype2);
		queryParams.put("etype", etype);
		return countByNamedQuery(
				"Element.getCountByTypeAndUnion2PropertyTypesAndStringsLike",
				queryParams);
	}

	@SuppressWarnings("unchecked")
	public List<Element> findElementsByArgs(final String etype,
			final Map<String, String> args,
			final Map<String, Map<String, String>> childargs) {
		String rev = new Long(Modification.MaxRevision).toString();
		Criteria crit = createCriteria(Element.class, "e")
				.createAlias("references", "refs").createAlias("refs.reflist", "rlist")
				.createAlias("rlist.elementlist", "elist")
				.createAlias("elementType", "t").add(Restrictions.eq("t.name", etype));
//				.add(Restrictions.eq("e.state", LifecycleState.ACTIVE));

		logger.debug("TYPE: " + etype);

		if (args != null) {
			for (String key : args.keySet()) {
				String value = args.get(key);
				logger.debug("  {}: {}", key, value);
				DetachedCriteria q = DetachedCriteria
						.forClass(Element.class, "pve_" + key)
						.add(Restrictions.eqProperty("pve_" + key + ".id", "e.id"))
						.setProjection(
								org.hibernate.criterion.Property.forName("pve_" + key + ".id"));

				DetachedCriteria propCriteria = q.createCriteria("properties");
				propCriteria.createCriteria("type").add(Restrictions.eq("name", key));
				DetachedCriteria pvlCriteria = propCriteria.createCriteria("valuelist")
						.add(Restrictions.eq("nextRevision", Modification.MaxRevision))
						.createAlias("values", "v")
						.add(Restrictions.like("v.svalue", value).ignoreCase());
				crit.add(Subqueries.exists(q));
			}
		}
		/*
		 * Criteria elementRefsCriteria = crit.createCriteria("references").
		 * add(Restrictions.eq("nextRevision",History.MaxRevision));
		 * 
		 * Criteria elementListCriteria=elementRefsCriteria.
		 * createCriteria("reflist").createCriteria("elementlist", "r");
		 */

		if (childargs != null) {
			for (String type : childargs.keySet()) {

				DetachedCriteria subquery = DetachedCriteria
						.forClass(Element.class, "e_" + type)
						.add(Restrictions.eqProperty("e_" + type + ".id", "elist.id"))
						.setProjection(Projections.id());

				Map<String, String> subargs = childargs.get(type);
				logger.debug("child type {}", type);

				for (String key : subargs.keySet()) {
					DetachedCriteria propCriteria = subquery.createCriteria("properties");
					propCriteria
							.createAlias("type", "t")
							.add(Restrictions.eq("t.name", key))
							.createAlias("valuelist", "vl")
							.add(Restrictions.eq("nextRevision", Modification.MaxRevision))
							.createAlias("vl.values", "v")
							.add(Restrictions.like("v.svalue", subargs.get(key)).ignoreCase());
					logger.debug("  {} --> {}", key, subargs.get(key));
				}
				// elementRefsCriteria.add(Restrictions.eq("refname", type));
				crit.add(Subqueries.exists(subquery));
			}
		}
		DetachedCriteria revisions = 
		  DetachedCriteria.forClass(Modification.class, "m");
	 revisions.add( Restrictions.eqProperty("m.element.id", "e.id")).
	 setProjection( Projections.property("m.nextRevision") );
	 crit.add(Subqueries.in(Modification.MaxRevision, revisions));
	 return crit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY).list();
	}

	public List<Element> getAllActive() {
		DetachedCriteria revisions = 
			  DetachedCriteria.forClass(Modification.class, "m");
		 revisions.add( Restrictions.eqProperty("m.element.id", "e.id")).
		 setProjection( Projections.property("m.nextRevision") );
		 
		Criteria crit = createCriteria(Element.class, "e")
		.add(Subqueries.in(Modification.MaxRevision, revisions));

		return crit.list();
	}
}
