package ch.semafor.gendas.interceptors;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.lang.time.StopWatch;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *  Time measurement of method calls using Spring AOP
 * @author John Briscoe
 *
 */
public class MethodTimingInterceptor implements MethodInterceptor {

	private final Logger log = LoggerFactory.getLogger(getClass());

	public Object invoke(final MethodInvocation methodInvocation)
			throws Throwable {
		final StopWatch stopWatch = new StopWatch();
		stopWatch.start();
		Object methodResult = null;
		try {
			methodResult = methodInvocation.proceed();
		} finally {
			stopWatch.stop();
		}
		final long millis = stopWatch.getTime();

		if (log.isInfoEnabled()) {
			log.info("{}.{} Total Time {} ms",
					new Object[]{methodInvocation.getThis().getClass().getName(),
					methodInvocation.getMethod().getName(), millis} );
		}

		return methodResult;
	}

}
