/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.semafor.gendas.dao.ElementDao;

@Entity
@Table(name = "elementrefs_lists")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class ElementRefList implements Serializable {
	@Transient
	final private Logger logger = LoggerFactory.getLogger(ElementRefList.class);
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id; // NOPMD by wim on 9/20/10 1:28 PM

	@Column(nullable = false)
	private Long revision;

	@Column(nullable = false)
	private Long nextRevision;

	@ManyToOne
	@JoinColumn(name = "elementrefs_id")
	private ElementRefs parent; // NOPMD by wim on 9/20/10 2:49 PM

	@ManyToMany(cascade = { CascadeType.ALL })
	@JoinTable(name = "elementrefs_elements", joinColumns = @JoinColumn(name = "ref_id"), inverseJoinColumns = @JoinColumn(name = "element_id"))
	private List<Element> elementlist = null; // NOPMD by wim on 9/20/10 2:49 PM

	// Constructor
	public ElementRefList() {
		super();
	}

	// Constructor
	public ElementRefList(final ElementRefs refs) throws CoreException {
		super();
		// ToDo PMD: NullAssignment, code smell
		this.id = null;
		this.parent = refs;
		this.revision = getNewRevision();
		this.nextRevision = Modification.MaxRevision;
		this.parent.addElementRefList(this);
	}

	// Constructor
	public ElementRefList(final ElementRefList reflist, final ElementRefs refs)
			throws CoreException {
		super();
		// ToDo PMD: NullAssignment, code smell
		this.id = null;
		this.parent = refs;
		this.revision = getNewRevision();
		this.nextRevision = Modification.MaxRevision;
		reflist.setNextRevision(this.revision);
		this.parent.addElementRefList(this);
	}

	// getter and setter of Id
	public Long getId() {
		return id;
	}

	public void setId(final Long id) { // NOPMD by wim on 9/20/10 1:28 PM
		this.id = id;
	}

	// getter and setter of Revision
	public void setRevision(final long rev) {
		this.revision = rev;
	}

	public Long getRevision() {
		return this.revision;
	}

	// getter and setter of NextRevision
	public void setNextRevision(final long rev) {
		this.nextRevision = rev;
	}

	public Long getNextRevision() {
		return this.nextRevision;
	}

	// getter and setter of Property
	public void setElementRefs(final ElementRefs refs) {
		this.parent = refs;
	}

	public ElementRefs getElementRefs() {
		return this.parent;
	}

	// getter and setter of ElementList
	public void setElementList(final List<Element> ellist) {
		this.elementlist = ellist;
	}

	public List<Element> getElementList() {
		if (this.elementlist == null) {
			this.elementlist = new ArrayList<Element>();
		}
		return this.elementlist;
	}

	public boolean isTransient() {
		return this.id == null;
	}

	public boolean isPersistent() {
		return this.id != null;
	}

	public boolean isInRevision(final long rev) {
		if (rev == Modification.MaxRevision) {
			return this.nextRevision == rev; // NOPMD by wim on 9/20/10 2:48 PM
		}
		return rev >= this.revision && rev < this.nextRevision;
	}

	public boolean isLastRevision() {
		return this.nextRevision == Modification.MaxRevision;
	}

	// New Revision for current Transaction
	private long getNewRevision() throws CoreException {
		return this.parent.getNewRevision();
	}

	public void createNewRevision() throws CoreException {
		setNextRevision(this.parent.getNewRevision());
	}

	public boolean isEmpty() {
		return this.elementlist.isEmpty();
	}

	public boolean isntEmpty() {
		return !this.elementlist.isEmpty();
	}

	public void addElement(Element el) throws CoreException { // NOPMD by wim
																																// on 9/20/10
																																// 1:27 PM
		if (el == null) {
			throw new CoreException("Element pointer is null");
		}
		if (this.elementlist == null) {
			this.elementlist = new ArrayList<Element>();
		}
		final Long entityId = el.getId();
		if (entityId != null) {
			logger.debug("Checking {}", entityId);
			for (Element e : this.elementlist) {
				logger.debug(" {}", e.getId());
				if (e.getId().equals(entityId)) {
					logger.warn(
							"Element #{} (type {}) already included in ElementRefList",
							entityId, el.getElementType().getName());
					/*
					 * throw new CoreException("Element #"+entityId+
					 * " (type "+el.getElementType
					 * ().getName()+") already included in ElementRefList");
					 */
					return;
				}
			}
		}
		this.elementlist.add(el);
	}

	public void assign(final ElementRefList from, List<Element> knownElements,
			final ElementDao elementDao) throws CoreException {
		logger.debug("begin of ElementRefList::assign()");

		logger.debug("Known Elements: {}", knownElements.size());
		logger.debug("New Elements: {}", from.elementlist.size());

		final List<Element> toElementsNew = new ArrayList<Element>();

		boolean changed = false;
		for (Element fromElement : from.elementlist) {
			Element toElement = getMatchingElement(knownElements, fromElement);
			if (toElement != null) { // NOPMD by wim on 9/20/10 1:25 PM
				logger.debug(
						"getMatchingElement(): Element {} #{} found isLoaded {}",
						new Object[] { toElement.getTypeName(), toElement.getId(),
								toElement.isLoaded() });

				if (fromElement.isPersistent() && !toElement.isLoaded()) {
					toElement = elementDao.get(fromElement.getId());
					toElement.setLoaded(); // to prevent another load

					if (logger.isDebugEnabled()) {
						fromElement.print(10,
								"ElementRefList: get matching persistent element "
										+ fromElement.getTypeName());
					}
				}
				// toElement.assign(fromElement, knownElements, elementDao);
				toElementsNew.add(toElement);
			} else {
				logger.debug("getMatchingElement(): No match: new Element added");
				if (fromElement.isPersistent()) {
					final Element existing = elementDao.get(fromElement.getId());
					if (logger.isDebugEnabled()) {
						existing.print(10, "ElementRefList: get new persistent element "
								+ existing.getTypeName());
					}
					// existing.assign(fromElement, knownElements, elementDao);
					if (logger.isDebugEnabled()) {
						existing.print(
								10,
								"ElementRefList: after assign: persistent element "
										+ existing.getTypeName());
					}
					fromElement = existing;
				} else {
					// ToDo PMD: AvoidInstantiatingObjectsInLoops
					final Element new_el = new Element(fromElement.getElementType());
					knownElements.add(new_el);
					new_el.assign(fromElement, knownElements, elementDao);
					fromElement = new_el;
				}
				toElementsNew.add(fromElement);
				changed = true;
			}
		}
		if (knownElements.size() > 0) {
			logger.debug("remaining old elements: {}", knownElements.size());
			changed = true;
		}
		ElementRefList dest_reflist = this;
		if (changed) {
			if (isPersistent()) {
				logger.debug("Change detected: create new ElementRefs from #{}",
						this.getId());
				dest_reflist = new ElementRefList(this, this.parent);
			}
		}
		dest_reflist.elementlist = toElementsNew;
		logger.debug("end of ElementRefs::assign()");
	}

	private Element getMatchingElement(final List<Element> elementlist,
			final Element src_el) {
		logger.debug("getMatchingElement: {}  Elements in list", elementlist.size());
		for( Element e: elementlist) {
			if (e.getElementType().equals(src_el.getElementType())) {
				/*
				 * TODO check this ! if (src_el.getId() != null) { if
				 * (src_el.getId().equals(el.getId())) {
				 */
				if (src_el.equals(e)) {
					logger.debug("getMatchingElement: same entityId => match");
					return e; 
				}

			} else {
				logger.debug("getMatchingElement: not of same type");
			}
		}
		return null;
	}

	public boolean crunch() throws CoreException {
		logger.debug("begin of ElementRefList::crunch()");

		// Elements
		if (this.elementlist != null) {
			Iterator<Element> ei = this.elementlist.iterator(); // NOPMD by wim on
																													// 9/20/10 1:30 PM
			while (ei.hasNext()) {
				Element el = ei.next(); // NOPMD by wim on 9/20/10 1:30 PM
				if (el.crunch()) {
					ei.remove();
				}
			}
		}
		logger.debug("end of ElementRefList::crunch()");
		if (this.elementlist != null) {
			return isEmpty(); // NOPMD by wim on 9/20/10 2:48 PM
		}
		return true; // no list
	}

	// marks a list as invalid
	public void delete() throws CoreException {
		if (isTransient()) {
			throw new CoreException(
					"it is not allowed to delete a new ElementRefList");
		}
		if (getNextRevision() != Modification.MaxRevision) {
			throw new CoreException(
					"it is not allowed to delete an already deleted ElementRefList");
		}
		createNewRevision();
	}

	public String toString() {
		ToStringBuilder sb = new ToStringBuilder(this, // NOPMD by wim on 9/20/10
																										// 1:30 PM
				ToStringStyle.SHORT_PREFIX_STYLE);
		sb.append("id", this.id);
		sb.append("revision", revision + "-" + nextRevision);
		sb.append("size", this.elementlist.size());
		for (Element e : this.elementlist) {
			sb.append(e.getTypeName());
		}
		return sb.toString();
	}

	private void indent(final int ind) {
		this.parent.indent(ind);
	}

	public void print(final int ind) {
		ToStringBuilder sb = new ToStringBuilder(this, // NOPMD by wim on 9/20/10
																										// 1:30 PM
				ToStringStyle.SHORT_PREFIX_STYLE);
		sb.append("id", this.id);
		sb.append("revision", revision + "-" + nextRevision);
		if (this.elementlist == null) {
			sb.append("elementlist is null");
		} else {
			sb.append("size", this.elementlist.size());
		}
		indent(ind);
		// ToDo really needed, PMD: SystemPrintln
		System.out.println(sb.toString());
		if (this.elementlist != null) {
			if (this.elementlist.size() > 0) { // NOPMD by wim on 9/20/10 1:33 PM
				int index = 0;
				for (Element e : this.elementlist) {
					index++;
					e.print(ind + 1, "Reference #" + index);
				}
			}
		}
	}

	// compares two objects
	public boolean equals(Object o) { // NOPMD by wim on 9/20/10 1:30 PM
		if (this == o) {
			return true; // NOPMD by wim on 9/20/10 2:48 PM
		}
		if (!(o instanceof ElementRefList)) {
			return false; // NOPMD by wim on 9/20/10 2:48 PM
		}
		final ElementRefList reflist = (ElementRefList) o;
		return equalsElementList(reflist.getElementList());
	}

	/**
	 * check if lists are equal
	 * 
	 * @param that_ellist
	 * @return true if lists are equal
	 */
	public boolean equalsElementList(final List<Element> that_ellist) {
		final List<Element> this_ellist = this.elementlist;

		if (this_ellist == null) {
			if (that_ellist == null) {
				return true; // NOPMD by wim on 9/20/10 2:49 PM
			}
			return false; // NOPMD by wim on 9/20/10 2:49 PM
		} else {
			if (that_ellist == null) {
				return false; // NOPMD by wim on 9/20/10 2:49 PM
			}
		}
		if (this_ellist.size() != that_ellist.size()) {
			return false; // NOPMD by wim on 9/20/10 2:49 PM
		}
		final Iterator<Element> this_it = this_ellist.iterator();
		final Iterator<Element> that_it = that_ellist.iterator();

		while (this_it.hasNext()) {
			final Element this_el = this_it.next();
			final Element that_el = that_it.next();
			if (!this_el.equals(that_el)) {
				return false; // NOPMD by wim on 9/20/10 2:49 PM
			}
		}
		return true;
	}

	// get hash-code
	public int hashCode() {
		final int prime = 31; // NOPMD by wim on 9/20/10 1:27 PM
		int result = 1;
		result = prime * result
				+ (this.elementlist != null ? elementlist.hashCode() : 0); // NOPMD by
																																		// wim on
																																		// 9/20/10
																																		// 1:25 PM
		return result;
	}

	// for debugging
	public boolean checkParent(final ElementRefs refs) {
		return this.parent != refs;
	}

	public void setLoaded() {
		if (elementlist == null) {
			return;
		}
		for (Element e : elementlist) {
			e.setLoaded();
		}
	}

	public void resetPrinting() {
		for (Element e : elementlist) {
			e.resetPrinting();
		}
	}

	public ElementRefs getParent() {
		return this.parent;
	}

	public List<Element> getElementlist() {
		return this.elementlist;
	}

}
