/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.hibernate.annotations.MapKeyManyToMany;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Entity
@Table(name = "element_types")
public class ElementType implements Serializable {
  private static final long serialVersionUID = 1L;

  @Id
  @GeneratedValue(strategy = GenerationType.AUTO)
  private Long id; // NOPMD by wim on 9/20/10 1:54 PM

  @Column(unique = true, nullable = false)
  private String name;

  private String beanId;
  private String beanVersionId;
  
  @ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST,
      CascadeType.REMOVE })
  @JoinTable(name = "element_types_property_types")
  // @org.hibernate.annotations.Cascade(value = {
  // org.hibernate.annotations.CascadeType.DELETE_ORPHAN})
  private List<PropertyType> propertyTypes = new ArrayList<PropertyType>(); // NOPMD by wim on 9/20/10 1:55 PM

  @MapKeyManyToMany
  @ManyToMany(cascade = CascadeType.MERGE) 
  @JoinTable(name = "element_type_references")
  Map<String,ElementType> references = new HashMap<String, ElementType>(); 
  /*
  List<ElementType> references = new ArrayList<ElementType>(); 
  */

 
  @Transient
  private boolean created = false; // break recursion in recursive loop
  @Transient
  private boolean printed = false; // break recursion in recursive loop // NOPMD by wim on 9/20/10 1:54 PM

  // Constructor
  public ElementType() {
  }

  // Constructor
  public ElementType(final String name) {
    this.name = name;
  }

  // Constructor
  public ElementType(final String name, final Integer aggregation) {
    this.name = name;
  }

  // getter and setter of Id
  public Long getId() {
    return id;
  }

  public void setId(final Long id) { // NOPMD by wim on 9/20/10 1:54 PM
    this.id = id;
  }

  public void setCreated() {
    created = true;
  }

  public boolean isCreated() {
    return created || id != null;
  }

  // getter and setter of Name
  public String getName() {
    return name;
  }

  public void setName(final String name) {
    this.name = name;
  }

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object o) { // NOPMD by wim on 9/20/10 1:54 PM
    if (this == o) {
      return true; // NOPMD by wim on 9/20/10 1:53 PM
    }
    if (!(o instanceof ElementType)) {
      return false; // NOPMD by wim on 9/20/10 1:54 PM
    }

    final ElementType t = (ElementType) o; // NOPMD by wim on 9/20/10 1:54 PM
    return !(name != null ? !name.equals(t.getName()) : t.getName() != null); // NOPMD by wim on 9/20/10 1:54 PM
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    return (name != null ? name.hashCode() : 0); // NOPMD by wim on 9/20/10 1:54 PM
  }

  public void add(final PropertyType propType) {
    if (propertyTypes.contains(propType)) {
      return;
    }
    propertyTypes.add(propType);
  }

  // get PropertyType by Name
  public PropertyType getPropertyType(final String type) {
    for (PropertyType p : propertyTypes) {
      if (p.getName().equals(type)){
        return p; // NOPMD by wim on 9/20/10 1:53 PM
      }
    }
    return null;
  }

  // get list of PropertyTypes
  public List<PropertyType> getPropertyTypes() {
    return propertyTypes;
  }

  public void addReference(final String refName, final ElementType ref) {
   final Logger logger = LoggerFactory.getLogger(ElementType.class); 
	logger.debug("ElementType {} <--- {} ( of {})", new Object[] {getName(), ref.getName(), references.size()});
	if( !references.containsKey(refName)){
		references.put(refName, ref);
	}
	/*
    if (!references.contains(ref)) { // NOPMD by wim on 9/20/10 1:54 PM
      references.add(ref);
    } else {
      logger.debug("    IGNORED");
    }*/
  }

  /**
   * check if reference is already included
   * 
   * @param refName name of reference to check
   * @return true if reference exist false otherwise
   */
  public boolean hasReference(final String refName) {
  	return references.containsKey(refName);
  	/*
    for (ElementType r : references) {
      if (r.name.equals(canonicalName)) {
        return true; // NOPMD by wim on 9/20/10 1:53 PM
      }
    }
    return false; */
  }

 

  public String toString() {
    final ToStringBuilder sb = new ToStringBuilder(this, // NOPMD by wim on 9/20/10 1:54 PM
        ToStringStyle.SHORT_PREFIX_STYLE);
    sb.append("id", this.id);
    sb.append("name", this.name);
    if (!printed) {
      printed = true;
     
      if (this.references.size() > 0) {
        sb.append("References: [");
        for (String refname : this.references.keySet()) {
          sb.append(refname);
        }
        sb.append("]");
      }
    }
    return sb.toString();
  }

  /*
  public List<ElementType> getReferences() {
    return this.references;
  }*/

  public String getBeanId() {
	return beanId;
}

  public void setBeanId( String beanId ) {
		this.beanId = beanId;
	}

  public String getBeanVersionId() {
		return beanVersionId;
	}

  public void setBeanVersionId( String beanVersionId ) {
	  this.beanVersionId = beanVersionId;
  }

  public boolean isBeanId(String propName) {
		return propName!=null && beanId!=null && beanId.equals(propName);
  }
  public boolean isBeanVersionId(String propName) {
	  return propName!=null && beanVersionId!=null && beanVersionId.equals(propName);
  }
}