/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

@Entity
@Table(name="modifications")
public class Modification implements Serializable {
	public static final long MaxRevision=9999999L;
  @Id
  @GeneratedValue(strategy = GenerationType.AUTO)
  private Long id; // NOPMD by wim on 9/20/10 2:03 PM
 
  @ManyToOne
  @JoinColumn(name = "element_id")
  private Element element;

  @Column( nullable=false )
  private Long revision;

  @Column( nullable=false )
  private Long nextRevision;
  
  @Column(nullable=false,updatable=false)
  @Temporal(TemporalType.TIMESTAMP)
  private Date timestamp; // NOPMD by wim on 9/20/10 2:03 PM
  
  // Constructor
  public Modification(){
  }
  
  // Constructor
  public Modification( final Element el ) throws CoreException { // NOPMD by wim on 9/20/10 2:03 PM
    //ToDo check, PMD: NullAssignment
	this.id            = null;
    this.element       = el;
    final Modification last_history = el.getLastModification();
    if( last_history == null ){
      this.revision = 1L;
    }
    else
    if( last_history.isTransient() ){
      throw new CoreException( "New Revision of History exists already" );
    }
    else{
      this.revision      = last_history.getRevision() + 1;
    }
    this.nextRevision = MaxRevision;
    this.timestamp     = new Date();
  }

  // getter and setter of Id
  public Long getId() {
    return this.id;
  }
  public void setId(final Long id) { // NOPMD by wim on 9/20/10 2:03 PM
    this.id = id;
  }

  public boolean isTransient(){
    return this.id == null;  
  }

  public boolean isPersistent(){
    return this.id != null;  
  }
  
  // getter and setter of Element
  public Element getElement() {
    return element;
  }
  public void setElement(final Element el) { // NOPMD by wim on 9/20/10 2:03 PM
    this.element = el;
  }

  // setter and getter of Revision
  public void setRevision( final long rev ){
    this.revision = rev;
  }
  public Long getRevision(){
    return this.revision;
  }

  // NextRevision
  public void setNextRevision( final long rev ){
    this.nextRevision = rev;
  }
  public Long getNextRevision(){
    return this.nextRevision;
  }
  
  // Timestamp
  public Date getTimestamp(){
    return this.timestamp;
  }

  public String toString() {
	final ToStringBuilder sb = new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE); // NOPMD by wim on 9/20/10 2:03 PM
    sb.append( "id", this.id );
    sb.append( "revision", revision + " -> " + nextRevision );
    sb.append( "timestamp", this.timestamp );
    return sb.toString();
  }
  
  public void print( final int ind ){
    element.indent( ind );
    System.out.println( toString() ); // NOPMD by wim on 9/20/10 2:04 PM
  }
  
  public boolean checkParent( final Element e ){ // NOPMD by wim on 9/20/10 2:05 PM
    return this.element == e;
  }
}
